
/*
 * Put the uv-coverage in a fits file
 * argv[1] must be the name of the configfile
 * argv[2] must be the name of the output fits file
*/


#include "APO.h"
#include "lib/ann_0.2/include/ANN/ANN.h"



APO apo1;
Grid grid1;

Instrume alma;
Obs obs1;
Transf transf1;
Site Chaj;
Model mod1;
Configfile filei;
Padslist pads;

//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------
void getArgs(int argc, char **argv);		// get command-line arguments

char namei[MAXSTRING], uvfits[MAXSTRING], dir[MAXSTRING]; 
//int diamfrac;

/*==========================================*/


void main(int argc, char **argv){

  int i,j,k,l,natot,ok,nconf;
  int withpauses;
  long npix;
  char pause;
  float dec, hamin, dt, obsdur;
  float bslmax, gridrmax0, gridrmin0, gridsz0, diam, du;
  float bsl, gridrmax, gridrmin;
  char s[15];


  printf("----Start of main------ \n");
  getArgs(argc, argv);

  gridsz0=6;
  withpauses=0;

 
  filei=namei;
  filei.read(&alma, &Chaj, &obs1, &mod1);
  obs1.get(&dec, &hamin, &dt, &obsdur, &nconf);
  natot=alma.getnatot();
  bslmax=alma.getdmax();
  diam=alma.getdiam(0);
  gridrmax0=mod1.getdmax();
  gridrmin0=mod1.getdmin();
  dt=12.*diam/(PI*bslmax);
  printf("dt =%f mn obsdur/dt=%f\n",dt*3600.,obsdur/dt);
  obs1.set(dec, &hamin, dt, &obsdur, 1);
  alma.print();
  du=diam;
  if(withpauses) scanf("%c",&pause);


  grid1.set(gridsz0, gridrmin,gridrmax, &mod1);
  printf("grid %d set\n",i);
  apo1.set(&alma, &obs1, &transf1, &Chaj, &grid1);

  npix=long (2*bslmax/du+1);
  if (npix%2 > 0) npix=npix+1;
  printf("number of pixels=%ld \n",npix);
  apo1.writeuvcov(uvfits,npix,du);
  printf("----End of main------ \n");
}

//----------------------------------------------------------------------

void getArgs(int argc, char **argv)
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  corr_uvcov2fits [-d directory] [-i inputfile] [-f fitsfile] [-du diamfrac]\n\n"
	<< "  where:\n"
	<< "    inputfile       name of the configfile to be used as input\n"
	<< "    fitsfile        name of the fitsfile where to write the uvcoverage\n"
	<< "    dir             directory where input and fitsfile are (opt)\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}
	else if (!strcmp(argv[i], "-i")) {	// -i option
	  sprintf(namei,"%s",argv[++i]);        // input file
	}
	else if (!strcmp(argv[i], "-f")) {		// -f option
	  sprintf(uvfits,"%s",argv[++i]);         // fits file
	}
        else {					// illegal syntax
	    cerr << "Unrecognized option.\n";
	    exit(1);
	}
	i++;
    }
    if (strlen(uvfits)==0 || strlen(namei)==0 ) {
	cerr << "-f and -i options must be specified\n";
	exit(1);
    }

    if (strlen(dir)!=0){
      strcpy(tmp,dir);
      strcat(tmp,namei);
      strcpy(namei,tmp);
      strcpy(tmp,dir);
      strcat(tmp,uvfits);
      strcpy(uvfits,tmp);
    }
}
