#include "APO.h"

#define GRIDSZ0 8
#define NGRIDS0 9
#define NITER 100
#define FA0 0.06
#define DPTACC0 0.1
#define NDEC 1
#define DEC0 -53.0
#define DECSHAD -63.0

//APO vapo[NGRIDS0];
APO vapo[NGRIDS0*NDEC];
Grid vgrid[NGRIDS0];
Obs vobs[NDEC];
Transf vtransf[NDEC];
Instrume alma;
Site Chaj;
Model mod1;
Configfile filei, fileo;

//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------
void getArgs(int argc, char **argv);		// get command-line arguments

char namei[MAXSTRING], nameo[MAXSTRING], dir[MAXSTRING]; 
//----------------------------------------------------------------------


void main(int argc, char **argv){

  int i,j,k,l,iter,natot;
  int test1,test2;
  char pause;
  float fa,gsh,dptacc;
  float gridrmax, gridrmin, dmax,fwhm;
  char *stdname="stdev.data";
  time_t rtime;
  FILE *fp;
  float stdev,integr;

  printf("----Start of main------ \n");
  getArgs(argc, argv);

  rtime=time(NULL);
  if((fp=fopen(stdname,"w")) == NULL){
    printf("ERROR: cannot open file %s\n",stdname);
    exit(1);
  }

  filei=namei;
  fileo=nameo;
  filei.read(&alma, &Chaj, vobs, &mod1);
  //scanf("%c",&pause);
  //alma.print();
  //Chaj.print();

  gsh=-1e10*FA0;

  natot=alma.getnatot();
  gridrmax=mod1.getdmax();
  gridrmin=mod1.getdmin();
  dmax=alma.getdmax();

  printf("gridrmax=%f\n",gridrmax);
  fwhm=mod1.getfwhm()/dmax;
  printf("fwhm=%f\n",fwhm);


  for(i=0;i<NGRIDS0;i++){
    vgrid[i].set(GRIDSZ0+2*i, gridrmin,gridrmax, &mod1);
    printf("grid %d set\n",i);
  }
  vobs[0].set(DEC0);
  for(i=1;i<NDEC;i++){
    vobs[i]=vobs[0];
    vobs[i].set(DEC0+i*30);
    printf("obs %d set\n",i);
  }
  for(i=0;i<NGRIDS0;i++){
    for(j=0;j<NDEC;j++){
      vapo[j*NGRIDS0+i].set(&alma, vobs+j, vtransf+j, &Chaj, vgrid+i);
    }
  }
  vobs[0].print();
  //scanf("%c",&pause);
  
  vtransf[0].setshadow(DECSHAD);

  //vapo[0].correct(dptacc);
  correct(vapo,NGRIDS0*NDEC,DPTACC0);
  //for(i=0;i<NGRIDS0;i++) vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
  //fileo.write(&alma, &Chaj, &obs1, &mod1);

  //scanf("%c",&pause);
  for(i=0;i<NGRIDS0;i++) {
    stdev=vapo[i].getstdev();
    fprintf(fp,"%-9.2f ",stdev);
  }
  fprintf(fp,"\n");
  for(k=0;k<NITER;k++) {
    printf("k=%d\n",k);
    fa=FA0/5.;
    if(k>2) fa=FA0/3;
    if(k>5) fa=FA0;
    if(k>NITER-10) fa=FA0/1.5;
 
    dptacc=DPTACC0;
    if(k<5) dptacc=5*DPTACC0;
    if(k<15) dptacc=2*DPTACC0;
    optimize(vapo,NGRIDS0*NDEC,fa,DPTACC0);
    if(k>10 && k<NITER-20) shake(vapo,NGRIDS0*NDEC,dptacc);

    for(i=0;i<NGRIDS0;i++) {
      stdev=vapo[i].getstdev();
      fprintf(fp,"%-9.2f ",stdev);
    }
    fprintf(fp,"\n");
  }
  fileo.write(&alma, &Chaj,vobs, &mod1);
  rtime=time(NULL)-rtime;
  printf("total runing time= %ld sec\n",rtime);
  fclose(fp);
}
//----------------------------------------------------------------------

void getArgs(int argc, char **argv)
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  cpct [-d directory] [-i inputfile] [-o outputfile] \n\n"
	<< "  where:\n"
	<< "    inputfile       name of the configfile to be used as input\n"
	<< "    outputfile      name of the configfile to be used as output\n"
	<< "    dir             directory where input file is read and output file written\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}
	else if (!strcmp(argv[i], "-i")) {	// -i option
	  sprintf(namei,"%s",argv[++i]);        // input file
	}
	else if (!strcmp(argv[i], "-o")) {		// -f option
	  sprintf(nameo,"%s",argv[++i]);         // fits file
	}
        else {					// illegal syntax
	    cerr << "Unrecognized option.\n";
	    exit(1);
	}
	i++;
    }
    if (strlen(nameo)==0 || strlen(namei)==0 ) {
	cerr << "-i and -o options must be specified\n";
	exit(1);
    }
    if (strlen(dir)!=0){
      strcpy(tmp,dir);
      strcat(tmp,namei);
      strcpy(namei,tmp);
      strcpy(tmp,dir);
      strcat(tmp,nameo);
      strcpy(nameo,tmp);
    }
}

