
#include "APO.h"
#include <string.h>

#define GRIDSZ0 7
#define NGRIDS0 7
#define NITER 50
#define FA0 0.045
#define FF0 0.1
#define DPTACC0 10.0
#define NLOADMAX 5
#define ICONFIG 5


//APO vapo[NGRIDS0];
APO vapo[NGRIDS0];
Grid vgrid[NGRIDS0];
Obs obs1;
Transf transf1;
Instrume alma;
Site Chaj;
Model mod1;
Configfile filei, fileo;
Padslist pads;
//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------
void getArgs(int argc, char **argv);		// get command-line arguments

char namei[MAXSTRING], nameo[MAXSTRING], dir[MAXSTRING]; 
//----------------------------------------------------------------------


void main(int argc, char **argv){

  int i,j,k,l,iter,natot,nloaded;
  int test1,test2;
  char pause;
  float fa,gsh,dptacc,ff;
  float gridrmax, gridrmin, bsl,fwhm;
  float **traject;
  char namepadsi[MAXSTRING], namepadso[MAXSTRING];
  time_t rtime;
  float integr;
  char trajfile[MAXSTRING];
  FILE  *trajfp; 

  printf("----Start of main------ \n");
  getArgs(argc, argv);
  sprintf(namepadsi,"%s%s",dir,"arraypads_0.data");
  sprintf(namepadso,"%s%s",dir,"arraypads.data");
  sprintf(trajfile,"%s%s%d%s",dir,"array",ICONFIG+1,"_traj.data");
  rtime=time(NULL);

  filei=namei;
  fileo=nameo;
  pads=namepadsi;
  filei.read(&alma, &Chaj, &obs1, &mod1);
  pads.read();
  pads.print();
  transf1.set(&obs1, &Chaj);
  //scanf("%c",&pause);
  //alma.print();
  //Chaj.print();

  natot=alma.getnatot();
  traject=matrix(2*(NITER+1),natot);
  alma.getpositions(traject[0], traject[1]);

  gridrmax=mod1.getdmax();
  gridrmin=mod1.getdmin();
  //dmax=alma.getdmax();
  bsl=alma.getdmax();
  //printf("gridrmax=%f\n",gridrmax);
  //fwhm=mod1.getfwhm()/dmax;
  //printf("fwhm=%f\n",fwhm);


  for(i=0;i<NGRIDS0;i++){
    vgrid[i].set(GRIDSZ0+i, gridrmin,gridrmax, &mod1);
    printf("grid %d set\n",i);
  }
  printf("ok\n");

  for(i=0;i<NGRIDS0;i++)
    vapo[i].set(&alma, &obs1 , &transf1, &Chaj, vgrid+i);

  obs1.print();
  scanf("%c",&pause);
  

  correct(vapo,NGRIDS0,dptacc);

  scanf("%c",&pause);
  //alma.freeants();
  ff=FF0;
  for(k=0;k<NITER;k++) {
    printf("k=%d\n",k);
    fa=FA0/10.;
    if(k>3) fa=FA0/3;
    if(k>6) fa=FA0;
    if(k>NITER-10) fa=FA0/1.5;
 
    dptacc=DPTACC0;
    if(k<10) dptacc=20*DPTACC0;
    if(k<20) dptacc=10*DPTACC0;
    if(k<30) dptacc=5*DPTACC0;
    
    if(k>20 && k<35 && k%2==1){
      nloaded=alma.loadants2pads(&pads, NLOADMAX, bsl, ff, ICONFIG);
      if(nloaded>0){
	for(i=0;i<NGRIDS0;i++) 
	  vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
	correct(vapo,NGRIDS0,DPTACC0);
      }
    }
    
    optimize(vapo,NGRIDS0,fa,dptacc);
    alma.getpositions(traject[2*(k+1)], traject[2*(k+1)+1]);
    if(k>10 && k<39) shake(vapo,NGRIDS0,dptacc);

  }
  if((trajfp=fopen(trajfile,"w")) == NULL){
    printf("ERROR: cannot open file %s\n",trajfile);            
    exit(1);
  }
  for(i=0;i<natot;i++){
    for(j=0;j<NITER+1;j++) 
      fprintf(trajfp,"%-8.1f %-8.1f ",traject[2*j][i],traject[2*j+1][i]);
    fprintf(trajfp,"\n");
  }
  fclose(trajfp);
  delete_matrix(traject);
  fileo.write(&alma, &Chaj,&obs1, &mod1);
  alma.addants2pads(&pads, ICONFIG);
  pads.write(namepadso);
  rtime=time(NULL)-rtime;
  printf("total runing time= %ld sec\n",rtime);
}
//----------------------------------------------------------------------

void getArgs(int argc, char **argv)
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  extended [-d directory] [-i inputfile] [-o outputfile] \n\n"
	<< "  where:\n"
	<< "    inputfile       name of the configfile to be used as input\n"
	<< "    outputfile      name of the configfile to be used as output\n"
	<< "    dir             directory where input file is read and output file written\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}
	else if (!strcmp(argv[i], "-i")) {	// -i option
	  sprintf(namei,"%s",argv[++i]);        // input file
	}
	else if (!strcmp(argv[i], "-o")) {		// -f option
	  sprintf(nameo,"%s",argv[++i]);         // fits file
	}
        else {					// illegal syntax
	    cerr << "Unrecognized option.\n";
	    exit(1);
	}
	i++;
    }
    if (strlen(nameo)==0 || strlen(namei)==0 ) {
	cerr << "-i and -o options must be specified\n";
	exit(1);
    }
    if (strlen(dir)!=0){
      strcpy(tmp,dir);
      strcat(tmp,namei);
      strcpy(namei,tmp);
      strcpy(tmp,dir);
      strcat(tmp,nameo);
      strcpy(nameo,tmp);
    }
}

