/*
 * Creates the file containing the scientific goals that will be read by
 * optimizearray.cpp.
 * argv[1] must be the name of starting configuration (the compact one).
 * argv[2] gives the name of the file to be created for the scientific goals.
*/

#include "APO.h"

#define PI 3.14159
#define OBSDURMAX 8.
#define BMIN 200.
#define NCONFI 5 // # of config 

#define ALPHA1 4.
#define BETA1 1.
#define GAMMA1 15.
#define LAMBDA1 3.e-3 //wavelength in mm
#define BSLMAX 3000. //limit for baseline set to 0 if no limit


Instrume alma;
Obs obs1;
Transf transf1;
Site Chaj;
Model mod1;
Configfile filei;

main(int argc, char **argv){
 
  int i, ok=1, iconfig=0, na;
  float Bmax1, diam, dB, cst;
  float Bl1,reso1;
  float tab[NCONFI][6];
  char ScGoalsfile[MAXSTRING];
  FILE *fp;

  filei=argv[1];
  sprintf(ScGoalsfile,"%s",argv[2]);
  filei.read(&alma, &Chaj, &obs1, &mod1);

  cst=2.*sqrt(log(10)/10.*log(2.))/PI;

  na=alma.getna();
  diam=alma.getdiam(0);
  Bmax1=na*(na-1)*ALPHA1*diam*OBSDURMAX/(48.*1.44);
  if(BSLMAX!=0.){
    printf("\nWARNING: there is a limit for the baseline length:\n");
    printf("         BSLMAX=%f\n\n",BSLMAX);
    if(Bmax1>BSLMAX) Bmax1=BSLMAX;     
  }
  printf("Bmax1=%f\n",Bmax1);
  dB=(Bmax1-BMIN)/(NCONFI-1);
  for(i=0;i<NCONFI;i++){
    Bl1=BMIN+i*dB;
    reso1=cst*sqrt(GAMMA1)*LAMBDA1*180*3600/(PI* BETA1 * Bl1);
    tab[i][0]=Bl1;
    tab[i][1]=ALPHA1;
    tab[i][2]=BETA1;
    tab[i][3]=GAMMA1;
    tab[i][4]=reso1;
    tab[i][5]=LAMBDA1/1.e-3;


    printf("Bl1=%f reso1=%f\n", Bl1, reso1);
  }
  //qsort(tab,2*NCONFI,sizeof(float *),compvec);
  if((fp=fopen(ScGoalsfile,"w")) == NULL){
    printf("ERROR: cannot open file %s\n",ScGoalsfile);            
    exit(1);
  }  
  fprintf(fp,"! alpha beta gamma(dB) dtheta(arcsec) lambda(mm)\n");
  for(i=0;i<NCONFI;i++){
  
    fprintf(fp,"%-3.1f %-3.1f %-4.1f %-7.3e %-3.1f\n",
	   tab[i][1],tab[i][2],tab[i][3],tab[i][4],tab[i][5],tab[i][6]);
  }
  fclose(fp);

}

