/*
 * This routine is aimed at optimizing compact configurations for
 * a range of maximum baselines and writing the final standard
 * deviations in the file stdev.data. The quality of the configurations
 * can thus be compared and a compromize be chosen between
 * compactness and quality.

 */
#include "APO.h"
#include <string.h>
#define ngrids 7
#define NCPCT 15

APO vapo[ngrids];
Grid vgrid[ngrids];
Obs obs1;
Transf transf1;
Instrume alma;

Site Chaj;
Model mod1;


void main(){

  int i,j,k,l,iter,niter, natot, icpct;
  char s[15];
  int test1,test2;
  char pause;
  float fa0,fa1,fa,dptacc;
  float gridrmax, gridrmin, gridsz0,dmax,fwhm, fwhm_dmax;
  char  namei[MAXSTRING], nameo[MAXSTRING],dir[MAXSTRING], stdname[MAXSTRING];
  time_t rtime;
  FILE *fp;
  Configfile filei, fileo;
  float stdev,integr;
  printf("----Start of main------ \n");
  rtime=time(NULL);
  sprintf(dir,"%s","config/multi_cpct_15dB/");
  sprintf(stdname,"%s%s",dir,"stdev.data");
  sprintf(namei,"%s%s",dir,"initconf.data");
  filei=namei;
  filei.read(&alma, &Chaj, &obs1, &mod1);
  //scanf("%c",&pause);
  //alma.print();
  //Chaj.print();

  fa0=0.005;
  natot=alma.getnatot();
  gridrmax=mod1.getdmax();
  gridrmin=mod1.getdmin();
  fwhm=mod1.getfwhm();
  dmax=alma.getdmax();
  //fwhm=1./0.9113*dmax; //for 10dB
  fwhm=1./1.117*dmax; //for 15dB
  mod1.set(fwhm,gridrmin,gridrmax);

  for(icpct=0;icpct<NCPCT;icpct++){
    sprintf(nameo,"%s%s%d%s",dir,"cpct15db_",icpct,".data");
    fileo=nameo;
    fa1=fa0+(icpct*0.02)*fa0;
    if(icpct>0){
      gridrmax+=10.;
      dmax=gridrmax;
      //fwhm=1./0.9113*dmax;//for 10dB
      fwhm=1./1.117*dmax; //for 15dB
    }
    mod1.set(fwhm,gridrmin,gridrmax);
    alma.setvdmax( &dmax, 1);
    printf("gridrmax=%f\n",gridrmax);
    fwhm_dmax=fwhm/dmax;
    printf("fwhm_dmax=%f\n",fwhm_dmax);

    if(fwhm_dmax>1){
      if(fwhm_dmax<1.8) fa1/=(5*(pow(fwhm_dmax,1.4)-0.8));
      else fa1/=(5*(pow(1.8,1.4)-0.8));
    }
    printf("fa1=%f\n",fa1);
    dptacc=0.1; 
    gridsz0=7;
    for(i=0;i<ngrids;i++){
      vgrid[i].set(gridsz0+2*i, gridrmin,gridrmax, &mod1);
      vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);  
      printf("grid %d set\n",i);
    }

    obs1.print();
    //scanf("%c",&pause);

    correct(vapo,ngrids,dptacc);
    niter=40;
    for(k=0;k<niter;k++) {
      printf("k=%d\n",k);
      if(k==0) fa=fa1;
      else fa=3*fa1;
      if(k>4) fa=10*fa1;
      if(k>14) fa=8*fa1;
      if(k>29) fa=3*fa1;
      //    if(k>niter-5) fa=5*fa1;
      if(k>niter-3) fa=fa1;
      //fa=fa1;
      optimize(vapo,ngrids,fa,dptacc);
      if(k<niter-3) shake(vapo,ngrids,dptacc);
    }
    fileo.write(&alma, &Chaj,&obs1, &mod1);
    printf("total runing time= %ld sec\n",rtime);
    if((fp=fopen(stdname,"a")) == NULL){
      printf("ERROR: cannot open file %s\n",stdname);
      exit(1);
    }
    fprintf(fp,"! %s %-9.1f %-9.1f\n",nameo,dmax,fwhm);
    for(i=0;i<ngrids;i++) {
      stdev=vapo[i].getstdev();
      fprintf(fp,"%-9.2f ",stdev);
    }
    fprintf(fp,"\n");
    fclose(fp);
  }
  rtime=time(NULL)-rtime;
}

