#include "APO.h"
#include <string.h>

#define GRIDSZ0 5
#define NGRIDS0 8
#define NITER 50
#define FA0 0.04
#define DPTACC0 1.0

APO vapo[NGRIDS0];
Instrume alma;
Obs obs1;
Transf transf1;
Site Chaj;
Grid vgrid[NGRIDS0];
Model mod1;
Configfile filei, fileo;

//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------

char namei[MAXSTRING], nameo[MAXSTRING], dir[MAXSTRING]; 
void getArgs(int argc, char **argv);   // get command-line arguments

/*==========================================*/

void main(int argc, char **argv){

  int i,j,k,l,iter, natot, ngrids;
  int test1,test2;
  int withpauses;
  char pause;
  float fa,dptacc;
  float gridrmax, gridrmin, dmax,fwhm;
  char *stdname="stdev.data",s[15];
  time_t rtime;
  FILE *fp;
  float stdev,integr;

  printf("----Start of main------ \n");
  getArgs(argc, argv);
  withpauses=0;

  rtime=time(NULL);
  if((fp=fopen(stdname,"w")) == NULL){
    printf("ERROR: cannot open file %s\n",stdname);
    exit(1);
  }
  filei=namei;
  fileo=nameo;
  filei.read(&alma, &Chaj, &obs1, &mod1);
  alma.print();
  //alma.sort();
  //alma.print();
  //fileo.write(&alma, &Chaj, &obs1, &mod1); 
  if(withpauses){
    printf("press any key to continue");
    scanf("%c",&pause);
  }
  //alma.print();
  //Chaj.print();

  natot=alma.getnatot();
  dmax=alma.getdmax();

  fwhm=mod1.getfwhm()/dmax;
  printf("fwhm=%f\n",fwhm);
 
  // if(fwhm>1){
  // if(fwhm<1.8) FA0/=(5*(pow(fwhm,1.4)-0.8));
  // else FA0/=(5*(pow(1.8,1.4)-0.8));
  //}
  printf("FA0=%f\n",FA0); 

  gridrmax=mod1.getdmax();
  gridrmin=mod1.getdmin();
  printf("gridrmax=%f\n",gridrmax);
  for(i=0;i<NGRIDS0;i++){
    vgrid[i].set(GRIDSZ0+i, gridrmin,gridrmax, &mod1);
    printf("grid %d set\n",i);
    vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
  }

  //transf1.setshadow(-73.);

  //correct(vapo,NGRIDS0,DPTACC,0);
  correct(vapo,NGRIDS0,DPTACC0);

  //fileo.write(&alma, &Chaj, &obs1, &mod1);
  //vapo[0].optimize(FA0,DPTACC);
  if(withpauses){
    printf("press any key to continue");
    scanf("%c",&pause);
  }
  for(i=0;i<NGRIDS0;i++) {
    stdev=vapo[i].getstdev();
    fprintf(fp,"%-9.2f ",stdev);
  }
  fprintf(fp,"\n");
  for(k=0;k<NITER;k++) {
    printf("k=%d\n",k);

    fa=FA0/200;
    if(k>2) fa=FA0/100;
    if(k>3) fa=FA0/50;
    if(k>5) fa=FA0/25;
    if(k>7) fa=FA0/2;
    if(k>10) fa=FA0/1.5;
    if(k>13) fa=FA0;
    ;if(k>33) fa=FA0/1.2;
    ;if(k>43) fa=FA0/1.5;
    if(k>NITER-3) fa=FA0/1.5;       

    dptacc=DPTACC0;
    if(k<10) dptacc=10*DPTACC0;
    if(k<20) dptacc=5*DPTACC0;

    ngrids=NGRIDS0;
    if(k==NITER/2){
      for(i=0;i<ngrids;i++){
	vgrid[i].set(GRIDSZ0+i+1, gridrmin,gridrmax, &mod1);
	printf("grid %d set\n",i);
	vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
      }
    }
    if(k==NITER-12){
      ngrids=NGRIDS0-2;
      for(i=0;i<ngrids;i++){
	vgrid[i].set(GRIDSZ0+i+4, gridrmin,gridrmax, &mod1);
	printf("grid %d set\n",i);
	vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
      }
    }
    optimize(vapo,ngrids,fa,dptacc);
    if(k<NITER-10) shake(vapo,ngrids,dptacc);


    for(i=0;i<ngrids;i++) {
      stdev=vapo[i].getstdev();
      fprintf(fp,"%-9.2f ",stdev);
    }
    fprintf(fp,"\n");
  }
  fileo.write(&alma, &Chaj, &obs1, &mod1);
  rtime=time(NULL)-rtime;
  printf("total runing time= %ld sec\n",rtime);
  fclose(fp);
}

//----------------------------------------------------------------------

void getArgs(int argc, char **argv)
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  optimize [-d directory] [-i inputfile] [-o outputfile] \n\n"
	<< "  where:\n"
	<< "    inputfile       name of the configfile to be used as input\n"
	<< "    outputfile      name of the configfile to be used as output\n"
	<< "    dir             directory where input file is read and output file written\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}
	else if (!strcmp(argv[i], "-i")) {	// -i option
	  sprintf(namei,"%s",argv[++i]);        // input file
	}
	else if (!strcmp(argv[i], "-o")) {		// -f option
	  sprintf(nameo,"%s",argv[++i]);         // fits file
	}
        else {					// illegal syntax
	    cerr << "Unrecognized option.\n";
	    exit(1);
	}
	i++;
    }
    if (strlen(nameo)==0 || strlen(namei)==0 ) {
	cerr << "-i and -o options must be specified\n";
	exit(1);
    }
    if (strlen(dir)!=0){
      strcpy(tmp,dir);
      strcat(tmp,namei);
      strcpy(namei,tmp);
      strcpy(tmp,dir);
      strcat(tmp,nameo);
      strcpy(nameo,tmp);
    }
}
