
/*
 * Optimize an array
 * Each optimized configuration will be written in a file named array*.data
 * where * is a number from 2 to NCONFIG. The optimization goes from the
 * most compact to the most extended configuration.
 * A log is written in apolog.
*/


#include "APO.h"
#define GRIDSZ0 6
#define NGRIDS0 8
#define NCONFIG 5
#define NITER 100
#define FA0 0.047
#define FF0 2.2
#define DPTACC0 2.0
#define NLOADMAX 4

APO vapo[NGRIDS0];
Grid vgrid[NGRIDS0];

Instrume alma;
Obs obs1;
Transf transf1;
Site Chaj;
Model mod1;
Configfile filei, fileo;
Padslist pads;

void getArgs(int argc, char **argv);	
float getgainfactor(float iconfig, float k);
float getdptacc(float iconfig, float k);
float getcriticaldist(float iconfig, float k);
//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------
	// get command-line arguments

char dir[MAXSTRING];

/*==========================================*/


void main(int argc, char **argv){

  int i,j,k,l,iter, iconfig,natot,ok;
  int test1,test2;
  int nloaded, withpauses, ngrids;
  char pause;
  float fa, dptacc, ff, nfixed;
  float bsl0, gridrmax0, gridrmin0,  diam, obsdurmax;
  float bsl, gridrmax, gridrmin, fwhm;
  float alpha, obsdur, dt, dec, hamin;
  float vbsl[NCONFIG-1],vobsdur[NCONFIG-1],vfwhm[NCONFIG-1];
  float **traject;
  char s[15];
  char namei[MAXSTRING], nameo[MAXSTRING], logfile[MAXSTRING]; 
  char namepads[MAXSTRING], ScGoalsFile[MAXSTRING];
  char trajfile[MAXSTRING];
  FILE *logfp, *trajfp;  
  time_t now, rtime;

  printf("----Start of main------ \n");
  getArgs(argc, argv);
  rtime=time(NULL);
  obsdurmax=8.0;
  withpauses=0;
  sprintf(namei,"%s%s",dir,"array1.data");
  sprintf(namepads,"%s%s",dir,"arraypads.data");
  sprintf(ScGoalsFile,"%s%s",dir,"ScGoals.data");
  sprintf(logfile,"%s%s",dir,"apolog");

  if((logfp=fopen(logfile,"a")) == NULL){
    printf("ERROR: cannot open file %s\n",logfile);            
    exit(1);
  }          
  now=time(NULL);
  fprintf(logfp,"%s Starting array optimization \n",ctime(&now));
  fprintf(logfp,"   number of configurations=%d\n",NCONFIG);
  fprintf(logfp,"   maximum allowed observation duration %-5.2f\n",obsdurmax);
  fprintf(logfp,"   The Scientific goals are defined in=%s  \n",ScGoalsFile);
  fprintf(logfp,"   The pads positions are written in=%s  \n",namepads);
  fprintf(logfp,"   The first config is in namei=%s  \n",namei);
  fclose(logfp);

  filei=namei;
  pads=namepads;
  pads.setnconf(NCONFIG);
  filei.read(&alma, &Chaj, &obs1, &mod1);
  alma.offset(100.,0.);
  alma.freeants();
  alma.print();
  alma.addants2pads(&pads,0);
  pads.print();
  if(withpauses) scanf("%c",&pause);

  natot=alma.getnatot();
  traject=matrix(2*(NITER+1),natot);
  bsl0=alma.getdmax();
  diam=alma.getdiam(0);
  gridrmax0=mod1.getdmax();
  gridrmin0=mod1.getdmin();


  dec=-10;
  transf1.set(&obs1, &Chaj);

  if(withpauses) scanf("%c",&pause);

  for(iconfig=1;iconfig<NCONFIG;iconfig++)
    ok=alma.getBSL_FWHM_ObsDur(obsdurmax, ScGoalsFile, iconfig, 
		     &vbsl[iconfig-1], &vfwhm[iconfig-1], &vobsdur[iconfig-1]);
  printf("---------------------------\n");
  printf("The array will be optimized for the following parameters:\n");
  for(iconfig=1;iconfig<NCONFIG;iconfig++)
    printf("%d bsl=%-8.1f fwhm=%-8.1f obsdur=%-5.2f\n",
	   iconfig, vbsl[iconfig-1], vfwhm[iconfig-1], vobsdur[iconfig-1]);
  printf("---------------------------\n");
  if(withpauses) scanf("%c",&pause);

  for(iconfig=1;iconfig<NCONFIG;iconfig++){

    sprintf(nameo,"%s%s%d%s",dir,"array",iconfig+1,".data");
    sprintf(trajfile,"%s%s%d%s",dir,"array",iconfig+1,"_traj.data");
    printf("outputfile=%s\n",nameo);
    fileo=nameo;

    if(iconfig==1) nfixed=4;
    if(iconfig==2) nfixed=3;
    if(iconfig>2) nfixed=2;

    alma.freeants();
    for(i=0;i<nfixed;i++) alma.fixant(i);

    bsl=vbsl[iconfig-1];
    fwhm=vfwhm[iconfig-1];
    obsdur=vobsdur[iconfig-1];

    if((logfp=fopen(logfile,"a")) == NULL){
      printf("ERROR: cannot open file %s\n",logfile);            
      exit(1);
    }          
    now=time(NULL);
    fprintf(logfp,"%s nameo=%s  bsl=%-7.1f fwhm=%-7.1f obsdur=%-5.2f\n",
	    ctime(&now), nameo,  bsl, fwhm, obsdur);
    fclose(logfp);

    //    scanf("%c",&pause);

    dt=obsdur/10.;
    //if (obsdur>2) dt=obsdur/10.;
    //else {
    //  if(obsdur>1) dt=obsdur/5.;
    //  else dt=0.15;
    //}

    hamin=-obsdur/2.;
    obs1.set(dec, &hamin, dt, &obsdur, 1); 

    gridrmax=bsl;
    printf("gridrmax=%f\n",gridrmax);
    gridrmin=gridrmin0;
    printf("gridrmin=%f\n",gridrmin);

    alma.setvdmax(&bsl,1);
    mod1.set(fwhm, gridrmin, gridrmax);
    //alma.random(&Chaj, &obs1, &mod1);
    alma.getpositions(traject[0], traject[1]);
    // alma.print();
    //scanf("%c",&pause);

    for(i=0;i<NGRIDS0;i++){
      vgrid[i].set(GRIDSZ0+i, gridrmin,gridrmax, &mod1);
      printf("grid %d set\n",i);
      vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
    }

    correct(vapo,NGRIDS0,DPTACC0);

    printf("FA0=%f\n",FA0);

    // scanf("%c",&pause);

    for(k=0 ; k<NITER ; k++) {

      printf("k=%d\n",k);
      fa=getgainfactor(iconfig, k);
      dptacc=getdptacc(iconfig,k);
      ff=getcriticaldist(iconfig,k);


      ngrids=NGRIDS0;
      if(k==NITER/2){
	for(i=0;i<ngrids;i++){
	  vgrid[i].set(GRIDSZ0+i+1, gridrmin,gridrmax, &mod1);
	  printf("grid %d set\n",i);
	  vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
	}
      }
      if(k==NITER-12){
	ngrids=NGRIDS0-2;
	for(i=0;i<ngrids;i++){
	  vgrid[i].set(GRIDSZ0+i+4, gridrmin,gridrmax, &mod1);
	  printf("grid %d set\n",i);
	  vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
	}
      }
      if(k>19 && k<NITER-3 && k%3==1){
	nloaded=alma.loadants2pads(&pads, NLOADMAX, bsl, ff, iconfig);
	//	alma.print();
	//scanf("%c",&pause);
	if(nloaded>0){
	  for(i=0;i<NGRIDS0;i++) 
	    vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i);
	  correct(vapo,NGRIDS0,DPTACC0);
	}
      }
      optimize(vapo,ngrids,fa,dptacc);
      alma.getpositions(traject[2*(k+1)], traject[2*(k+1)+1]);
      if(k>10 && k<NITER-10) shake(vapo,ngrids,dptacc);
    }
    if((trajfp=fopen(trajfile,"w")) == NULL){
      printf("ERROR: cannot open file %s\n",trajfile);            
      exit(1);
    }
    for(i=0;i<natot;i++){
     for(j=0;j<NITER+1;j++) 
       fprintf(trajfp,"%-8.1f %-8.1f ",traject[2*j][i],traject[2*j+1][i]);
     fprintf(trajfp,"\n");
    }
    fclose(trajfp);
    fileo.write(&alma, &Chaj, &obs1, &mod1);
    alma.addants2pads(&pads, iconfig);
    pads.write();
    alma.freeants();
    for(i=0;i<nfixed;i++) alma.fixant(i);
  }
  rtime=time(NULL)-rtime;
  printf("total runing time= %ld sec\n",rtime);
  if((logfp=fopen(logfile,"a")) == NULL){
    printf("ERROR: cannot open file %s\n",logfile);            
    exit(1);
  }          
  now=time(NULL);
  fprintf(logfp,"total time=%-5.2f\n",rtime);
  fprintf(logfp,"-------End of array optimization------ \n");
  fclose(logfp);
  delete_matrix(traject);
  printf("----End of main------ \n");
  
}

//----------------------------------------------------------------------

void getArgs(int argc, char **argv)
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  optimizearray [-d dir]\n\n"
	<< "  where:\n"
	<< "    dir      is the name of the directory in which the required files are.\n"
	<< "              required files: \n"
	<< "               1. ScGoals.data which contains the scientific goals\n"
 	<< "               2. array1.data which contains the compact configuration\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}

	i++;
    }
    if (strlen(dir)==0) {
	cerr << "-d option must be specified\n";
	exit(1);
    }

}

float getgainfactor(float iconfig, float k){
  float fa, fa00;
  
  if(iconfig==1) fa00=0.7*FA0;
  if(iconfig>1) fa00=FA0;
  if(iconfig>3) fa00=1.2*FA0;

  fa=fa00/150;
  if(k>2) fa=fa00/70;
  if(k>3) fa=fa00/20;
  if(k>5) fa=fa00/8;
  if(k>7) fa=fa00/3;
  if(k>10) fa=fa00/1.5;
  if(k>13) fa=fa00;
  if(k>NITER-7) fa=fa00/1.2;
  if(k>NITER-3) fa=fa00/1.5;       
  
  return fa;
}

float getdptacc(float iconfig, float k){
  float dptacc, dptacc00;

  if(iconfig==1) dptacc00=0.2*DPTACC0;
  if(iconfig>1) dptacc00=DPTACC0;
  if(iconfig>4) dptacc00=3*DPTACC0;
 
  dptacc=dptacc00;

  if(k<20) dptacc=2*dptacc00;
  if(k<10) dptacc=3*dptacc00;

  return dptacc;
}

float getcriticaldist(float iconfig, float k){
  float dcrit, dcrit0;

  dcrit0=FF0/1.5;
  if(iconfig>3) dcrit0=FF0;
 
  dcrit=dcrit0;
  if(k==31) dcrit=1.5*dcrit0;
  if(k==37) dcrit=1.5*dcrit0;
  //if(k>40) dcrit=dcrit0/3.;
  if(k>NITER-15) dcrit=dcrit0/2.;
  if(k>NITER-8) dcrit=dcrit0/4.;
  return dcrit;
}
