#include "APO.h"

#define GRIDSZ0 8 // Number of cells (along the radius) of 
                  // the smallest grid
#define NGRIDS0 4 // Number of grids
#define NITER 2   // Number of iterations
#define FA0 0.06  // gain factor relating the force 
                  // to the density gradient
#define DPTACC0 0.1 // accuracy of the positions in meter

APO vapo[NGRIDS0];    // a vector of NGRIDS0 APO-instantiations
Grid vgrid[NGRIDS0];  // instantiation of the NGRIDS0
Obs obs1;             // instantiation of one observation situation
Transf transf1;       // instantiation of one transformation from
                      // uv-plane to antenna plane
Instrume alma;        // instantiation of one instrument named alma
Site Chaj;            // instantiation of one site named Chaj
Model mod1;           // instantiation of one gaussian model
Configfile filei, fileo; // instantiation of 2 config files.

//----------------------------------------------------------------------
//  Parameters that are set in getArgs()
//----------------------------------------------------------------------

char namei[MAXSTRING], nameo[MAXSTRING], dir[MAXSTRING]; 

void getArgs(int argc, char **argv); // prototype of the function 
                                     // that handles the command
                                     // line arguments defined at
                                     // the end of the file

//----------------------------------------------------------------------

void main(int argc, char **argv){

  int i,j,k,l,iter,natot;
  char pause;
  float fa,dptacc;
  float gridrmax, gridrmin, dmax;

  printf("----Start of main------ \n");

  getArgs(argc, argv);  // get command-line arguments

  filei=namei;  // relate filei to the file which name is in namei
  fileo=nameo;  // relate fileo to the file which name is in nameo
  filei.read(&alma, &Chaj, &obs1, &mod1); // read the parameters
                                          // in the input configfile
                                          // and initialize the variables
                                          // alma, Chaj, obs1 and mod1.

  alma.print(); // print the content of alma
  Chaj.print(); // print the content of Chaj

  natot=alma.getnatot(); // set natot to the total number of antennas
  gridrmax=mod1.getdmax(); // set gridrmax to the outer radius of the grid
  gridrmin=mod1.getdmin(); // set gridmin to the inner radius of the grid
  dmax=alma.getdmax(); //set dmax to the larges baseline allowed

  printf("gridrmax=%f\n",gridrmax);  //print gridrmax

  for(i=0;i<NGRIDS0;i++){
    vgrid[i].set(GRIDSZ0+2*i, gridrmin,gridrmax, &mod1); //initialize the grids
    printf("grid %d OK\n",i);
    vapo[i].set(&alma, &obs1, &transf1, &Chaj, vgrid+i); //initialize vapo
  }

  correct(vapo,NGRIDS0, DPTACC0); // correct the configuration if it
                                  // does not satisfy the constraints

  for(k=0;k<NITER;k++) { // loop from 0 to NITER-1
    printf("k=%d\n",k);
    optimize(vapo,NGRIDS0,FA0,DPTACC0); // optimize the configuration
                                        // using the NGRIDS0 grids

  }
  fileo.write(&alma, &Chaj, &obs1, &mod1); // write the result in the ouput
                                           // configfile
  printf("----End of main------ \n");
}

//----------------------------------------------------------------------

void getArgs(int argc, char **argv)  // function 
                                     // that handles the command
                                     // line arguments
{
  char tmp[MAXSTRING];

    if (argc <= 1) {				// no arguments
  	cerr << "Usage:\n\n"
	<< "  cpct [-d directory] [-i inputfile] [-o outputfile] \n\n"
	<< "  where:\n"
	<< "    inputfile       name of the configfile to be used as input\n"
	<< "    outputfile      name of the configfile to be used as output\n"
	<< "    dir             directory where input file is read and output file written\n";
	exit(0);
    }
    int i = 1;
    while (i < argc) {				// read arguments
	if (!strcmp(argv[i], "-d")) {	      // -d option
	  sprintf(dir,"%s",argv[++i]);        // directory
	}
	else if (!strcmp(argv[i], "-i")) {	// -i option
	  sprintf(namei,"%s",argv[++i]);        // input config file
	}
	else if (!strcmp(argv[i], "-o")) {	 // -o option
	  sprintf(nameo,"%s",argv[++i]);         // output config file
	}
        else {					// illegal syntax
	    cerr << "Unrecognized option.\n";
	    exit(1);
	}
	i++;
    }
    if (strlen(nameo)==0 || strlen(namei)==0 ) {
	cerr << "-i and -o options must be specified\n";
	exit(1);
    }
    if (strlen(dir)!=0){
      strcpy(tmp,dir);
      strcat(tmp,namei);
      strcpy(namei,tmp);
      strcpy(tmp,dir);
      strcat(tmp,nameo);
      strcpy(nameo,tmp);
    }
}

